/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "oss.h"
#ifdef ALSA
#include "alsa.h"
#endif
#include "loopback.h"
#include "iodisk.c"
#include <sys/errno.h>
#include <stdlib.h>
#include "audio.h"

int (*select_device)(char *in_name,char *out_name);
int (*open_dsp)(int rate,int channel,int bits,int bsize)=NULL;
int (*close_dsp)(int dspfd)=NULL;
int (*set_source)(char source)=NULL;
int (*set_in_level)(int lev)=NULL;
int (*set_out_level)(int lev)=NULL;
/*int (*read_dsp)(int dspfd,char*buffer, int size)=NULL;
int (*write_dsp)(int dspfd,char*buffer, int size)=NULL;*/
char* (*get_read_audio_buffer)(void)=NULL;
char* (*get_write_audio_buffer)(void)=NULL;


int audio_notsupp()
{
	return(-EOPNOTSUPP);
}

int audio_alsa_init()
{
#ifdef ALSA
	select_device=audio_notsupp;
  	open_dsp=open_dsp_alsa;
  	close_dsp=close_dsp_alsa;
  	get_read_audio_buffer=get_read_alsa_buffer;
  	get_write_audio_buffer=get_write_alsa_buffer;
  	set_source=set_source_alsa;
  	set_in_level=set_in_level_alsa;
  	set_out_level=set_out_level_alsa;
  	return(0);
#else
	return(-22);
#endif
}

int audio_oss_init()
{
	select_device=audio_notsupp;
  	open_dsp=open_dsp_oss;
  	close_dsp=close_dsp_oss;
  	get_read_audio_buffer=get_read_oss_buffer;
  	get_write_audio_buffer=get_write_oss_buffer;
  	set_source=set_source_oss;
  	set_in_level=set_in_level_oss;
  	set_out_level=set_out_level_oss;
  	return(0);
}


int audio_loopback_init()
{
	select_device=audio_notsupp;
	open_dsp=open_dsp_loopback;
  	close_dsp=close_dsp_loopback;
  	get_read_audio_buffer=get_read_loopback_buffer;
  	get_write_audio_buffer=get_write_loopback_buffer;
  	set_source=set_source_loopback;
  	set_in_level=set_in_level_loopback;
  	set_out_level=set_out_level_loopback;
  	return(0);
}

int audio_iodisk_init()
{
	select_device=audio_select_device_iodisk;
	open_dsp=open_dsp_iodisk;
  	close_dsp=close_dsp_iodisk;
  	get_read_audio_buffer=get_read_iodisk_buffer;
  	get_write_audio_buffer=get_write_iodisk_buffer;
  	set_source=set_source_iodisk;
  	set_in_level=set_in_level_iodisk;
  	set_out_level=set_out_level_iodisk;
  	return(0);
}	
