/*

Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <sys/soundcard.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include "oss.h"
#include "audio.h"

#define OPTIMAL_BLKSIZE 512   /* the minimum (optimal) dsp block size*/

static int dsp_fd=0;
static audio_buffer_t rbuffer,wbuffer;
static int blocksize=OPTIMAL_BLKSIZE;  /* the minimal (optimal) dsp block size*/
static int bufsize;  /* the wanted size*/
static int total_length,real_length;

int open_dsp_oss(int rate,int channel,int bits,int qsize)
{
  int i, p,err;
  int min_size,cond;
  
  bufsize=qsize;
  while(blocksize<qsize) blocksize=blocksize*2;
  printf("Acceptable blocksize for query (%i) is %i\n",qsize,blocksize);
	/* check if dsp is openable */
	err=open("/dev/dsp",O_RDWR|O_NONBLOCK);
	if (err==-1)
	{
		printf("Cannot open audio device because it is busy.\n");
		return(-1);
	} else close(err);
	
	/* if qsize=0 returns immediately 0. This was just to see if dsp was openable*/
	if (qsize==0)
	{
		close(err);
		return(0);
	}
	
	err = open("/dev/dsp", O_RDWR);
  if (err == -1){
    fprintf(stderr,"Cannot open /dev/dsp.  Do you have permision?\n");
    return(err);
  } else { 
    dsp_fd=err;
    fprintf(stderr," dsp opened.\n");

    ioctl(err, SNDCTL_DSP_RESET, 0);

    p =  bits;  /* 16 bits */
    i =  ioctl(err, SNDCTL_DSP_SAMPLESIZE, &p);
    
    p =  channel;  /* 2 canaux */
    i += ioctl(err, SNDCTL_DSP_CHANNELS, &p);
    
    p =  rate;  /* khz*/
    i += ioctl(err, SNDCTL_DSP_SPEED, &p);

    	i += ioctl(err, SNDCTL_DSP_GETBLKSIZE, &min_size);

    	/* try to subdivide BLKSIZE to reach blocksize if necessary */
    	if (min_size>blocksize)
    	{
    		cond=1;
    		p=min_size/blocksize;
    		while(cond)
    		{
       		i=ioctl(err, SNDCTL_DSP_SUBDIVIDE, &p);
     			printf("SUB_DIVIDE said error=%i,errno=%i\n",i,errno);
     			if ((i==0) || (p==1)) cond=0;
     			else p=p/2;
     		}
      }
    
    i += ioctl(err, SNDCTL_DSP_GETBLKSIZE, &min_size);
    if (min_size>blocksize)
      { 
			printf("Warning: dsp block size set to %i.\n",min_size);
			blocksize=min_size;
      }
   printf("blocksize is %i.\n",blocksize);
    total_length=blocksize*(N_BUFS+1);
    real_length=blocksize*N_BUFS;
    
    rbuffer.abuffer=calloc(total_length,sizeof(char));
    rbuffer.rd_ptr=blocksize;
    rbuffer.wr_ptr=blocksize;
    wbuffer.abuffer=calloc(total_length,sizeof(char));
    wbuffer.rd_ptr=blocksize;
    wbuffer.wr_ptr=0;
    
    return(err);
  }		
}

int set_source_oss(char source)
{
  int i, p, mix_fd;
  
  if (source == 'c')
    p = 1 << SOUND_MIXER_CD;
  if (source == 'l')
    p = 1 << SOUND_MIXER_LINE;
  if (source == 'm')
    p = 1 << SOUND_MIXER_MIC;
  
  mix_fd = open("/dev/mixer", O_WRONLY, 0);
  i = ioctl(mix_fd, SOUND_MIXER_WRITE_RECSRC, &p);
  close(mix_fd);
  return(i);
}


int set_in_level_oss(int a)
{
  int p,mix_fd;
	p=(((int)a)<<8 | (int)a);
  mix_fd = open("/dev/mixer", O_RDWR, 0);
  ioctl(mix_fd,MIXER_WRITE(SOUND_MIXER_RECLEV), &p);
  close(mix_fd);
  return(0);
}

int set_out_level_oss(int a)
{
  int p,mix_fd;

  p=(((int)a)<<8 | (int)a);
  mix_fd = open("/dev/mixer", O_WRONLY, 0);
  ioctl(mix_fd,(MIXER_WRITE(SOUND_MIXER_PCM)), &p);
  close(mix_fd);
  return(0); 
}

int close_dsp_oss()
{
	if (dsp_fd!=0)
	{
  		close(dsp_fd);
  		blocksize= OPTIMAL_BLKSIZE;
  		dsp_fd=0;
 	}
  	return(0);
}

int read_dsp_oss(int dspfd,char*buffer, int size)
{
  return(read(dspfd,buffer,size));
}

int write_dsp_oss(int dspfd,char*buffer, int size)
{
  return(write(dspfd,buffer,size));
}

char *get_read_oss_buffer()
{
  char *ret_ptr;
  int bytesread;

  if ((rbuffer.wr_ptr-rbuffer.rd_ptr)<bufsize)
    {
      //fprintf(stderr,"write pointer = %i ",rbuffer.wr_ptr);
      //performs the real read from to the dsp device
      if (rbuffer.wr_ptr>=total_length)
	{
	  rbuffer.wr_ptr=blocksize; //return at the beginning
	  rbuffer.rd_ptr=rbuffer.rd_ptr-real_length;
	  // and copy data at the beginning
	  memcpy(rbuffer.abuffer,&rbuffer.abuffer[real_length],blocksize);
	}
      bytesread=read(dsp_fd,&rbuffer.abuffer[rbuffer.wr_ptr],blocksize);
      if (bytesread!=blocksize) fprintf(stderr,"Only %i bytes read.\n",bytesread);
      rbuffer.wr_ptr+=blocksize;
    }
      // and then returns the buffer
  ret_ptr=&rbuffer.abuffer[rbuffer.rd_ptr];
  //fprintf(stderr,"read pointer = %i \n",rbuffer.rd_ptr);
  rbuffer.rd_ptr+=bufsize;
  
  return(ret_ptr);
}

char *get_write_oss_buffer()
{
  char *ret_ptr;
  int bytesread;
  if ((wbuffer.wr_ptr-wbuffer.rd_ptr)>=blocksize)
    {
      //performs the real write to the dsp device
      bytesread=write(dsp_fd,&wbuffer.abuffer[wbuffer.rd_ptr],blocksize);
      if (bytesread!=blocksize) fprintf(stderr,"Only %i bytes written.\n",bytesread);
      wbuffer.rd_ptr+=blocksize;
      if (wbuffer.wr_ptr>=real_length)
	{
	  wbuffer.rd_ptr=0; //return at the beginning
	  wbuffer.wr_ptr=wbuffer.wr_ptr-real_length;
	  // and copy data at the beginning
	  memcpy(wbuffer.abuffer,&wbuffer.abuffer[real_length],blocksize);
	}
      
    }
  // and then returns the buffer
  ret_ptr=&wbuffer.abuffer[wbuffer.wr_ptr];
  //fprintf(stderr,"read pointer = %i \n",rbuffer.rd_ptr);
  wbuffer.wr_ptr+=bufsize;
  
  return(ret_ptr);
}
