/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <pthread.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "../audio/audio.h"
/* the aim of this file is to provide an easy way to produce a ring */

static int ring_cond=0;
static pthread_t dring_thread;
static char *ring_buffer=NULL;
static char *ringback_buffer=NULL;
static int ring_buflen=0;
static int ringback_buflen=0;

static int open_file(char *name,char **ringbuffer, int *ringlen);
static void *ring_thread(void *p);
static void *ringback_thread(void *p);

int open_ringfile(char *name)
{
	return(open_file(name,&ring_buffer,&ring_buflen));
}

int open_ringbackfile(char *name)
{
	return(open_file(name,&ringback_buffer,&ringback_buflen));
}



static int open_file(char *name,char **ringbuffer, int *ringlen)
{
    int fd,err;
    struct stat statfile;

    if (ring_cond!=0) return(-11); /* you must stop ringing before opening new */
    /* see if there is not ever something in the buffer */
    if (ringlen[0]!=0)
    {
        free(*ringbuffer);
        *ringbuffer=NULL;
        *ringlen=0;
    }
    fd=open(name,O_RDONLY);
    if (fd<0)
    {
        printf("Cannot open ring file %s.\n",name);
        return(-1);
    }
    err=fstat(fd,&statfile);
    if (err<0)
    {
        printf("Cannot read stats for ring file.\n");
        close(fd);
        return(-1);
    }
    if (statfile.st_size==0)
    if (err<0)
    {
        printf("Ring file is empty.\n");
        close(fd);
        return(-1);
    }
    *ringlen=statfile.st_size;
    *ringbuffer=(char*)malloc(*ringlen);
    if (*ringbuffer==NULL)
    {
        printf("Cannot allocate memory.\n");
        return(-1);
    }
    err=read(fd,*ringbuffer,*ringlen);
    if (err<*ringlen)
    {
        printf("Error loading ring file.\n");
        free(*ringbuffer);
        *ringbuffer=NULL;
        *ringlen=0;
        close(fd);
        return(-1);
    }
    close(fd);
    return(0);
}

int close_ringfile()
{
    if (ring_buflen!=0) free(ring_buffer);
    ring_buflen=0;
    return(0);
}

int close_ringbackfile()
{
    if (ringback_buflen!=0) free(ringback_buffer);
    ringback_buflen=0;
    return(0);
}


int start_ring()
{
    int err,bufsize=2048;
    if (ring_buflen==0) return(-1);  /* we have no ring loaded in the buffer */
    err=open_dsp(8000,1,16,bufsize);
    if (err==-1) return(-1);
    close_dsp();
    ring_cond=1;
    err=pthread_create(&dring_thread,NULL,&ring_thread,NULL);
    return(err);
}

int start_ringback()
{
    int err,bufsize=2048;
    if (ringback_buflen==0) return(-1);  /* we have no ring loaded in the buffer */
    err=open_dsp(8000,1,16,bufsize);
    if (err==-1) return(-1);
    close_dsp();
    ring_cond=1;
    err=pthread_create(&dring_thread,NULL,&ringback_thread,NULL);
    return(err);
}

int stop_ring()
{
    if (ring_cond==0) return(0);
    ring_cond=0;
    pthread_join(dring_thread,NULL);     /* it may block a long time*/
    return(0);
}

void *ring_thread(void *p)
{
	int err,bytes=0,bufsize=2048,count;
	char *outbuf,*inbuf;
  	
  	while(ring_cond)
  	{
  		err=open_dsp(8000,1,16,bufsize);
  		if (err!=0) printf("audio: could not open audio device.\n");
  		while((bytes<ring_buflen)&&(ring_cond))
  		{
  			outbuf=get_write_audio_buffer();
  			memcpy(outbuf,&ring_buffer[bytes],bufsize);
  			bytes+=bufsize;
  			inbuf=get_read_audio_buffer(); /* just for synchronisation*/
  		}
  		close_dsp();
  		bytes=0;
  		count=0;
  		while ((ring_cond) &&(count<100) )
  		{
  			usleep(10000);
  			count++;
  		}
  	}
  	
  	return(NULL);
}

void *ringback_thread(void *p)
{
	int err,bytes=0,bufsize=2048;
	char *outbuf,*inbuf;
	int count;
  	
  	while(ring_cond)
  	{
  		err=open_dsp(8000,1,16,bufsize);
  		if (err!=0) printf("audio: could not open audio device.\n");
  		while((bytes<ringback_buflen)&&(ring_cond))
  		{
  			outbuf=get_write_audio_buffer();
  			memcpy(outbuf,&ringback_buffer[bytes],bufsize);
  			bytes+=bufsize;
  			inbuf=get_read_audio_buffer(); /* just for synchronisation*/
  		}
  		close_dsp();
  		bytes=0;
  		count=0;
  		while ((ring_cond) &&(count<100) )
  		{
  			usleep(10000);
  			count++;
  		}
  	}
  	
  	return(NULL);
}
