/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <osip/sip.h>
#include <osip/smsg.h>

#include "OsipUA.h"
#include "utils.h"

list_t callleg_list={0,NULL};

/* returns 1 if call leg exists*/
int osip_call_leg_exists(OsipCallLeg *call)
{
	int pos;
	OsipCallLeg *callleg;

	for (pos=0;pos<callleg_list.nb_elt;pos++)
	{
		callleg=(OsipCallLeg*)list_get(&callleg_list,pos);
		if (callleg==call) return 1;
  	}
	return 0;
}

OsipCallLeg *osip_call_leg_find(sip_t *sipmsg)
{
	int pos=0;
	int cond=0;
	OsipCallLeg *call;

  	/*find an existant call-leg that may match  this new invite */
  	while( !list_eol(&callleg_list,pos)&&(!cond))
  	{
  		call=(OsipCallLeg*)list_get(&callleg_list,pos);
		if ( call_id_match(sipmsg->call_id,call->callid)==0) cond=1;
		else pos++;
	}
	if (cond)
	{
		/* an existing call-leg was found */
		return(call);
	}
	else return(NULL);	
}


/* find an existing call leg using a transaction id*/
OsipCallLeg *osip_call_leg_find_with_tid(int tid)
{
	int i;
	OsipCallLeg *call;
	for (i=0;i<callleg_list.nb_elt;i++)
	{
		call=(OsipCallLeg*)list_get(&callleg_list,i);
		if (tid==call->last_tid)
		return(call);
	}
	return(NULL);
}


/* respond to a request that is part from a call-leg : ex 200OK (INVITE)*/
void osip_call_leg_respond(OsipCallLeg *call, int code, char *body)
{
   int osiperr;
	char *reason,*name;
	char *apptype=NULL;
	sip_t *response;
	transaction_t *transaction;
	unsigned long int addr;
	struct hostent *host;
	via_t *via;
	sipevent_t *sipevent;
	OsipUA *ua=(OsipUA*) call->ua;
	int tid=call->last_tid;
	osip_t *config=ua->config;
	transaction = trn_findid(config->transactions, tid);

	msg_init(&response,&osiperr);
	reason = parser_getreason(code);

	if (body!=NULL) apptype=sgetcopy("application/sdp");

    if (1==msg_makereply(code,reason,
			 transaction->lastrequest,
			 response,
			 body, /*local_body*/
			 apptype, /*apptype */
			 0,&osiperr))
	{
		/* temp code*/
		/* we must check if first via has x.x.x.x address. If not, we must resolve it*/
		via=(via_t*)list_get(response->vias,0);
		if (via==NULL) printf( "Error getting first via!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
		name=via->host;
		if ((int)(addr = inet_addr(name)) == -1)
		{
		 	host=resolv(name);
		 	if (host==NULL)
		 	{
		 		msg_free(response);
		 		sfree(response);
		 		sfree(reason);
		 		sfree(body);
		 		return;
		 	}
		 	else
		 	{
		 		/* replace host name by ip4 addr */
		 		sfree(name);
		 		via->host=sgetcopy(inet_ntoa( ( (struct in_addr*)host->h_addr)[0] ) );
		 	}
		}
       /* the user agent MUST set a contact header in last RFC*/
   		{	
	       char *tmp = (char *)smalloc(60*sizeof(char)+strlen(ua->contact->url->username));
    		sprintf(tmp,"<sip:%s@%s:%i>",ua->contact->url->username,ua->ua_ip4addr,ua->ua_port);
    		msg_setcontact(response,tmp,&osiperr);
    	}
    	
		sipevent = evt_new_fromuser(response);
		sipevent->transactionid =  transaction->transactionid;

		fifo_add(transaction->transactionff,sipevent);

#ifdef WITH_4THREADS
		trn_dispatch(transaction);
#endif
	}
	else
	{
		trace(TRACE_LEVEL1,NULL,"<userctrl.c> Could not create response for current status\n");
		sfree(response);
		sfree(reason);
		if (apptype!=NULL)
	  	sfree(apptype);
		return ;
      }
	sfree(reason);
	if (apptype!=NULL)
      sfree(apptype);
}


/* Destroy a call-leg and remove it from ua list and stack list*/
int osip_call_leg_destroy(OsipCallLeg *call)
{
	int err=0;
	struct  _OsipUA *ua=call->ua;
	MediaDesc *desc;
	
	err=list_remove_el(&callleg_list,call);
	if (err<0)  trace(TRACE_LEVEL2,stdout,"<OsipUA.c> Could not remove call-leg from stack list !\n");
	err=list_remove_el(&ua->call_list,call);
	printf("Call leg is removed. It remains %i call legs in the ua list.\n",err);
	if (err<0)  trace(TRACE_LEVEL2,stdout,"<OsipUA.c> Could not remove call-leg from ua list !\n");
	if (err) return(-EFAULT);
	if( call->to!=NULL)
	{
		from_free(call->to);
		sfree(call->to);
	}
	if (call->callid!=NULL)
	{
		call_id_free(call->callid);
		sfree(call->callid);
	}
	if (call->remote_sdp!=NULL)
	{
		sdp_free( call->remote_sdp);
		sfree(call->remote_sdp);
	}
	/* free the media_list */
	if (call->media_list!=NULL)
	{
		list_free(call->media_list,LIST_ELEM_DESTRUCTOR(media_desc_destroy));
		sfree(call->media_list);
	}	
	if(call->sdp_caddr!=NULL) sfree(call->sdp_caddr);
	if (call->registrar!=NULL)
	{
		to_free(call->registrar);
		sfree(call->registrar);
	}
	sfree(call);
	return(err);
}


/* extract all media descriptors from a sdp_t, and find compatible profiles for each one*/
/* if compatible profile is found for ALL medias, return 0, else -1. The list of MediaDesc is stored in OsipCallLeg::media_list*/
int osip_call_leg_extract_media(OsipCallLeg *call,sdp_t *remote_sdp)
{
	list_t *media_list;
	mediad_t *md;
	MediaDesc *mrem,*mloc;
	OsipUA *ua=(OsipUA*)call->ua;
	int prof,error,i;
	char *dest_addr,*p;
	
	/* get c field and extract ip address*/
	dest_addr=smalloc(strlen(remote_sdp->c));
	sscanf(remote_sdp->c,"%*s %*s %s",dest_addr);
	p=strchr(dest_addr,'/');
	if (p!=NULL) *p='\0';
	call->sdp_caddr=dest_addr;
	
	media_list=smalloc(sizeof(list_t));
	list_init(media_list);
	for(i=0;i<remote_sdp->mediads->nb_elt;i++)
	{
		/* get all streams and their properties, and store them in a list of MediaDesc OsipCallLeg::media_list */
		printf("Get stream\n");
		md=(mediad_t*)list_get(remote_sdp->mediads,i);
		mrem=media_desc_new_from_mediad(md);
		if (mrem->type==STREAM_AUDIO)
		{
			prof=get_compatible_profile(ua->maudio->profile_list,mrem->profile_list);
			mloc=media_desc_new(STREAM_AUDIO);
			mloc->loc_port=ua->maudio->loc_port;
			mloc->rem_port=mrem->rem_port;
		}
		else if ( mrem->type==STREAM_VIDEO)
		{
			prof=get_compatible_profile(ua->mvideo->profile_list,mrem->profile_list);
			mloc=media_desc_new(STREAM_VIDEO);
			mloc->loc_port=ua->mvideo->loc_port;
			mloc->rem_port=mrem->rem_port;
		}
		media_desc_destroy(mrem);	
		mloc->rem_ip4addr=dest_addr;
		if (prof!=-1)
		{
			/* if we have found a compatible profile, then add the media descriptor to the list maintened by the call-leg*/
			media_desc_append_profile(mloc,prof);
			list_add(media_list,mloc,-1);
		}
		else
		{
			/* we get uncompatible profile : send an unsupported media type*/
			trace(TRACE_LEVEL2,stdout,"<OsipCallLeg.c> Uncompatible media type in incoming message.\n");
			media_desc_destroy(mloc);
			error=-1;
		}
		printf("End of get stream\n");
	}	
	call->media_list=media_list;
	return(error);
}	

MediaDesc *media_desc_new(int type)
{
	MediaDesc *md;
	md=(MediaDesc*)smalloc(sizeof(MediaDesc));
	memset(md,0,sizeof(MediaDesc));
	md->type=type;
	md->profile_list=smalloc(sizeof(list_t));
	list_init(md->profile_list);
	return(md);
}

MediaDesc *media_desc_new_from_char(char *list, int type)
{
	int err=2;
	int *profile;
	int prof=0,c;
	char *p=list;
	MediaDesc *md;
	
	md=(MediaDesc*)smalloc(sizeof(MediaDesc));
	md->type=type;
	md->profile_list=smalloc(sizeof(list_t));
	list_init(md->profile_list);
	while(err>0)
	{
		
		err=sscanf(p,"%i %n",&prof,&c);
		//printf("scanning..%i \n",err);
		if (err>0)
		{
			profile=(int*)smalloc(sizeof(int));
			profile[0]=prof;
			//printf("prof=%i c=%i   %s\n",prof,c,p);
			list_add(md->profile_list,profile,-1);
			p+=c;
			if (c==0) err=0;
		}
	}
	return(md);
}

void media_desc_append_profile(MediaDesc *md, int profile)
{
	int *prof;
	if (md->profile_list==NULL) return;
	
	prof=(int*)smalloc(sizeof(int));
	prof[0]=profile;
	list_add(md->profile_list,prof,-1);
}

char * media_desc_profiles_2char(MediaDesc *md)
{
	int i,c;
	int *prof;
	char *str,*p;
	list_t *proflist=md->profile_list;
	
	if (proflist==NULL) return(NULL);
	p=str=(char*)smalloc((5*proflist->nb_elt)+1);
	for (i=0;i<proflist->nb_elt;i++)
	{
		prof=(int*)list_get(proflist,i);
		sprintf(p,"%i %n",prof[0],&c);
		p+=c;
	}
	return(str);
}

char * media_desc_port_2char(MediaDesc *md)
{
	char *str;
	str=smalloc(6);
	sprintf(str,"%i",md->loc_port);
	return(str);
}
	
void media_desc_destroy(MediaDesc *md)
{
	int pos;
	int *el;
	/*
	for(pos=0;pos<md->profile_list->nb_elt;pos++)
	{
		el=list_get(md->profile_list,pos);
		if (el!=NULL) sfree(el);
	} */
	listofchar_free(md->profile_list);
	sfree(md);
}


/* get the list media streams with their profile when a call is starting*/
list_t * osip_call_leg_get_streams(OsipCallLeg *call_leg)
{
	if (call_leg->status!=CALL_RUNNING)
	{
		trace(TRACE_LEVEL2,stdout,"<OsipCallLeg.c> Bad Status:Call leg has not choosed its media for the moment.\n");
		return(NULL);
	
	}
	return(call_leg->media_list);
}

MediaDesc * media_desc_new_from_mediad(mediad_t *mediad)
{
	int c,err=1;
	char mtype[10];
	char *p;
	int type,port;
	MediaDesc *md;
	
	sscanf(mediad->m,"%s %i %*s %n",mtype,&port,&c);
	/*printf("*****************remote port = %i\n",port);*/
	p=mediad->m+c;
	if (strcasecmp(mtype,"audio")==0) type=STREAM_AUDIO;
	else if (strcasecmp(mtype,"video")==0) type=STREAM_VIDEO;
	else printf("Unknow stream type in mediad.\n");
	
	md=media_desc_new_from_char(p, type);
	md->rem_port=port;
	return(md);
}
	