/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef OSIP_CALL_LEG_H
#define OSIP_CALL_LEG_H

#include <osip/smsgtypes.h>
#include <osip/sdp.h>

#define STREAM_AUDIO 0
#define STREAM_VIDEO 1

/* this object is used to represent media streams used by user agent*/
typedef struct _MediaDesc
{
	int type;  /*can be STREAM_AUDIO or STREAM_VIDEO*/
	list_t *profile_list;
	int loc_port;    /* local rtp port*/
	int rem_port;   /* remote rt port*/
	char *rem_ip4addr;  /*remote ip address*/
#define profile_1(m) ((int*)(m->profile_list->node->element))[0]
} MediaDesc;

/*constructor*/
MediaDesc *media_desc_new(int type);
MediaDesc *media_desc_new_from_char(char *list, int type);

void media_desc_append_profile(MediaDesc *md, int profile);

char * media_desc_profiles_2char(MediaDesc *md);

char * media_desc_port_2char(MediaDesc *md);

MediaDesc * media_desc_new_from_mediad(mediad_t *mediad);

/*destructor*/
void media_desc_destroy(MediaDesc *md);

typedef struct _OsipCallLeg
{
	from_t *from;
	to_t *to;
	call_id_t *callid;
	int cseq_number;
	int status;
	to_t *registrar; /* if not NULL, INVITEs are sent to the registrar */
	
	/* transactions associated with call-leg */
	list_t incoming_transactions;
	list_t outgoing_transactions;
	int last_tid;    /* the id of the last transaction*/
	unsigned int expires; /* used in REGISTER requests*/

  /* all events for this transaction arrive here */
	fifo_t *calllegff;
	void *ua; /* the ua object that manages the call*/	
	
	/*sdp only parameters*/
	sdp_t *remote_sdp;
	int sdp_sessid;
	char* sdp_caddr;   /* the address in the c field*/
	/* list of used streams with their properties*/
	list_t *media_list;
} OsipCallLeg;


/* status for call-legs*/

#define CALL_NEW 0
#define CALL_INVITING 1
#define CALL_INVITED 2
#define CALL_RUNNING 3
#define CALL_TERMINATED 4
#define CALL_REGISTERING 101

/* the list of call-legs handled by the stack*/
extern list_t callleg_list;

/* returns 1 if call leg exists*/
int osip_call_leg_exists(OsipCallLeg *call);

/* find an existing call_leg in the stack list using an incmoing sip message. Returns NULL if not found*/
OsipCallLeg *osip_call_leg_find(sip_t *sipmsg);

/* find an existing call leg using a transaction id*/
OsipCallLeg *osip_call_leg_find_with_tid(int tid);

/* respond to a request that is part from a call-leg : ex 200OK (INVITE)*/
void osip_call_leg_respond(OsipCallLeg *call, int code, char *body);

/* Destroy a call-leg and remove it from ua list and stack list*/
int osip_call_leg_destroy(OsipCallLeg *call);

/* extract all media descriptors from a sdp_t, and find compatible profiles for each one*/
/* if compatible profile is found for ALL medias, return 0, else -1. The list of MediaDesc is stored in OsipCallLeg::media_list*/
int osip_call_leg_extract_media(OsipCallLeg *call,sdp_t *remote_sdp);

/* possible requests*/
#define INVITE 0
#define BYE 1
#define REGISTER 2
#define ACK 3




#endif