/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <errno.h>
#include <osip/sip.h>
#include <osip/sdp.h>
#include <OsipCallLeg.h>
#include <OsipUA.h>
#include "utils.h"

int OnEvent_connection_refused(int transactionid)
{
	OsipCallLeg *call;
	OsipUA *ua;
	int error=-ECONNREFUSED;
  trace(TRACE_LEVEL0,stdout,"<callback.c> OnEvent_connection_refused! \n");
	call=osip_call_leg_find_with_tid(transactionid);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> Connection refused for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*)call->ua;
	if (ua->faillure!=NULL)
		ua->faillure(call,&error);

  return 1; /* ok */
}

int OnEvent_network_error(int transactionid)
{
  OsipCallLeg *call;
	OsipUA *ua;
	int error=-errno;
  trace(TRACE_LEVEL0,stdout,"<callback.c> OnEvent_connection_refused! \n");
	call=osip_call_leg_find_with_tid(transactionid);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> Connection refused for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*)call->ua;
	if (ua->faillure!=NULL)
		ua->faillure(call,&error);

  return 1; /* ok */
}

int OnEvent_sndreq_retransmission(int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_sndreq_retransmission! \n");
  return 1; /* ok */
}

int OnEvent_rcvresp_retransmission(int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_rcvresp_retransmission! \n");
  return 1; /* ok */
}

int OnEvent_New_OutgoingInvite(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingInvite!\n");
  //uaapp_annouceoutinvite(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingAck(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingAck!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingBye(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingBye!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingRegister(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingRegister!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}


int OnEvent_New_OutgoingCancel(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingCancel!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingInfo(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingInfo!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingOptions(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingOptions!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingPrack(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingPrack!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_OutgoingUnknownRequest(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_OutgoingUnknownRequest!\n");
  //uaapp_annouceoutrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_Incoming1xxResponse(sipevent_t *sipevent,int transaction_id)
{
	OsipCallLeg *call;
	OsipUA *ua;
	int inf;
   char *strinf;
	
  	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming1xxResponse!\n");

	call=osip_call_leg_find_with_tid(transaction_id);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> 1xx response for an inexistant call leg! \n");
		return 1;
	}
	strinf=sipevent->sip->strtline->statuscode;
	if (strinf!=NULL) inf=satoi(strinf);
	ua=(OsipUA*)call->ua;
	if (ua->informative!=NULL)
		ua->informative(call,&inf);
	
  return 1; /* ok */
}

int OnEvent_New_Incoming2xxResponse(sipevent_t *sipevent,int transaction_id)
{
	sip_t *sipmsg=sipevent->sip;
  	sip_t *ack;
	OsipCallLeg *call;
	OsipUA *ua;
	int err,i;
	MediaDesc *desc;
	sdp_t *remote_sdp;
	transaction_t *trn;
   body_t *body;

	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming2xxResponse!\n");

  	call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		ua=(OsipUA*)(call->ua);
		trn=trn_findid( ua->config->transactions,transaction_id);
		if (trn==NULL)
		{
			trace(TRACE_LEVEL2,stdout,"<callback.c> Could not find transaction!!.\n");
			return 1;
		}
		if (MSG_IS_INVITE(trn->lastrequest))
		{
			call->status=CALL_RUNNING;
   			/* get first body, hoping it is a sdp*/
      		body=(body_t*)list_get(sipmsg->bodies,0);
        	if (body!=NULL)
         	{
          	/* we must check if there is a sdp, and check if it is compatible*/
           	remote_sdp=smalloc(sizeof(sdp_t));
			   sdp_init(remote_sdp);
			   err=sdp_parse(remote_sdp,body->body);
      			if (err==-1) sfree(remote_sdp);
			} else err=-1;
			if (err==-1)
			{
				trace(TRACE_LEVEL2,stdout,"<callbacks_uac.c> Syntax error in sdp or sdp not found.\n");
				 				/* we should return something here TODO (maybe BAD REQUEST)*/ 	
				return 1;
			}
			err=osip_call_leg_extract_media(call,remote_sdp);
			if (err==-1)
			{
				trace(TRACE_LEVEL2,stdout,"<callback.c> incompatible media description in 200 OK!!!!\n");
				return 1;
			}
			/* check if there is one profile for each media stream*/
			for (i=0;i<call->media_list->nb_elt;i++)
			{
				desc=(MediaDesc*) list_get(call->media_list,i);
				if (desc->profile_list->nb_elt==0)
				{
					trace(TRACE_LEVEL2,stdout,"<callback.c> No profile found in 200 OK!!!!\n");
					return 1;
				}
				else if (desc->profile_list->nb_elt>1)
				{
					trace(TRACE_LEVEL2,stdout,"<callback.c> Warning: more than one profile in 200 OK!!!!\n");
				}
			}
			/* indicate the user of the acceptation of the invite */
			if (ua->bye!=NULL)
				ua->invite_accepted(call,(void*)sipmsg);
			/* respond ACK to the server */
			ack=sip_compose_request(call,ACK,NULL);
			trace(TRACE_LEVEL2,stdout,"<callback.c> Sending ACK!\n");
			err=sip_send_request(call,ack);
		}
		else if (MSG_IS_REGISTER(trn->lastrequest))
		{
			/* call-legs initiated with REGISTER are very short*/
			int inf=200;
			if (ua->informative!=NULL)
				ua->informative(call,(void*)&inf);
			//trace(TRACE_LEVEL2,stdout,"<callback.c> Sending ACK!\n");
			osip_call_leg_destroy(call);
		}
	}
	else
	{
		trace(TRACE_LEVEL1,stdout,"<callback.c> 200 OK for an inexistant call-leg !\n");
		/* return an Inexistant Call-Leg*/
	}			
  return 1; /* ok */

}

int OnEvent_New_Incoming3xxResponse(sipevent_t *sipevent,int transaction_id)
{
	char *strinf,*tmp;
	int code,err;
	OsipCallLeg *call;
	OsipUA *ua;
	sip_t *ack;
	contact_t *contact=NULL;

	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming3xxResponse!\n");
	call=osip_call_leg_find_with_tid(transaction_id);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> 3xx response for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*) call->ua;
	strinf=sipevent->sip->strtline->statuscode;
	if (strinf!=NULL) code=satoi(strinf);
	ack=sip_compose_request(call,ACK,NULL);
	trace(TRACE_LEVEL2,stdout,"<callback.c> Sending ACK!\n");
	err=sip_send_request(call,ack);
	switch(code)
	{
	case 300: /*Multiples choices*/
		break;
	case 301: /*Moved permanently */
		break;
	case 302: /*Moved temporarily */
		trace(TRACE_LEVEL1,stdout,"<callback.c> User has moved temporarily.\n");
		/* get first contact field*/
		err=msg_getcontact(sipevent->sip,0, &contact);
		if (contact!=NULL)
		{  /* do redirection*/
			call->status=CALL_NEW;
			/* remove params like action=redirect*/
			listofchar_free(contact->from_params);
			contact_2char(contact,&tmp,&err);
			osip_call_leg_invite(call,tmp);
			sfree(tmp);
		}	
		break;
	case 305: /* Use proxy */
		break;
	}
	
	/* then we invite using the address in the contact header*/
	return 1; /* ok */
}

int OnEvent_New_Incoming4xxResponse(sipevent_t *sipevent,int transaction_id)
{
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming4xxResponse!\n");
	call=osip_call_leg_find_with_tid(transaction_id);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> 4xx response for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*)call->ua;
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	if (ua->faillure!=NULL)
		ua->faillure(call,&error);
	osip_call_leg_destroy(call);
  return 1; /* ok */
}

int OnEvent_New_Incoming5xxResponse(sipevent_t *sipevent,int transaction_id)
{
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming5xxResponse!\n");
	call=osip_call_leg_find_with_tid(transaction_id);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> 5xx response for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*)call->ua;
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	if (ua->faillure!=NULL)
		ua->faillure(call,&error);
	osip_call_leg_destroy(call);
  return 1; /* ok */
}

int OnEvent_New_Incoming6xxResponse(sipevent_t *sipevent,int transaction_id)
{	
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Incoming6xxResponse!\n");
	call=osip_call_leg_find_with_tid(transaction_id);
	if (call==NULL)
	{
		trace(TRACE_LEVEL2,stdout,"<callback.c> 6xx response for an inexistant call leg! \n");
		return 1;
	}
	ua=(OsipUA*)call->ua;
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	if (ua->faillure!=NULL)
		ua->faillure(call,&error);
	osip_call_leg_destroy(call);
  return 1; /* ok */
}
