/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <osip/sip.h>
#include <osip/sdp.h>
#include <OsipCallLeg.h>
#include <OsipUA.h>
#include "utils.h"


int OnEvent_sndresp_retransmission(int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_sndresp_retransmission!\n");
  return 1; /* ok */
}

int OnEvent_rcvreq_retransmission(int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_rcvreq_retransmission!\n");
  return 1; /* ok */
}

int OnEvent_New_IncomingInvite(sipevent_t *sipevent,int transaction_id)
{
	int err,i;
	int prof;
	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	sdp_t *remote_sdp;
      body_t *body;
	char *p;
   int osiperr;

  	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingInvite!\n");
  	call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		trace(TRACE_LEVEL1,stdout,"<callback.c> Receiving new invite for an existent call-leg!\n");
	}
	else
	{
		/* we must find a user agent interested by this new call */
		ua=osip_ua_find(sipmsg);
		if (ua!=NULL)
		{
               /* get first body, hoping it is a sdp.... more
things to do here*/
			body=list_get(sipmsg->bodies,0);
			 if (body!=NULL)
			{
			   /* check if we have an sdp in this invite*/
			   remote_sdp=smalloc(sizeof(sdp_t));
			   sdp_init(remote_sdp);
			   err=sdp_parse(remote_sdp,body->body);
      		} else err=-1;
			if (err==-1)
			{
				trace(TRACE_LEVEL2,stdout,"<callback.c> Syntax error in sdp or sdp not found.\n");
				/* we should return something here: maybe BAD REQUEST*/
				respond_to_request(ua->config, transaction_id, 400, NULL) ;
				return 1;
			}
			/* check if user agent has session properies */
			if ((ua->maudio->profile_list->nb_elt==0) && (ua->mvideo->profile_list->nb_elt==0))
			{
				trace(TRACE_LEVEL2,stdout,"<callback.c> User agent has NO session properties! Must reject an invite...\n");
				/* respond that there is an internal error*/
				respond_to_request(ua->config, transaction_id, 500, NULL) ;
				return(1);
			}
			/* send a 100 trying */
			trace(TRACE_LEVEL2,stdout,"<callback.c> Sending 100 trying.\n");
			respond_to_request(ua->config, transaction_id,100,NULL);
			/* check if the user agent can answer to the call */
			if (ua->call_list.nb_elt>=ua->max_calls)
			{
				/*respond a 486 busy here*/
				respond_to_request(ua->config, transaction_id,486,NULL);
				sdp_free(remote_sdp);
				sfree(remote_sdp);
				return 1;
			}
			call=osip_call_leg_new(ua);
			call->remote_sdp=remote_sdp;
			call->status=CALL_INVITED;
			callid_getcopy(call->callid,sipmsg->call_id);
			call->last_tid=transaction_id;
			from_2char(sipmsg->from,&p,&osiperr);
			from_parse(call->to,p,&osiperr);
			free(p);
			err=osip_call_leg_extract_media(call,remote_sdp);
			if (err==-1)
			{
				/*  send a 606 not acceptable response */
				trace(TRACE_LEVEL2,stdout,"<callback.c> Must reject invite because of incompatible media type.\n");
				osip_call_leg_respond(call,606,NULL);
				/* to do*/
				return 1;
			}
			trace(TRACE_LEVEL1,stdout,"<callback.c> Creating new call lag for an existing user-agent.\n");

			/* inform the user about this invite*/
			if (ua->invite!=NULL)
				err=ua->invite(call,(void*)sipmsg);
			/* if err==0 send 180 ringing since we are successfully informing the user...*/
			if (err==0) osip_call_leg_respond(call,180,NULL);
		}
		else
		{
			/* we should return a 404 Not Found*/
			trace(TRACE_LEVEL1,stdout,"<callback.c> Somebody requested an unknown user.\n");
			/* take the first user agent of the list to respond to this request  :  Maybe revisit*/
			ua=(OsipUA*)list_get(&ua_list,0);
			respond_to_request(ua->config, transaction_id, 404, NULL) ;
		}
	}			
  	return 1; /* ok */
}

int OnEvent_New_IncomingAck(sipevent_t *sipevent,int transaction_id)
{
  	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	
	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingAck!\n");

  	call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		ua=(OsipUA*)(call->ua);
		if (call->status==CALL_INVITED)
		{
			call->status=CALL_RUNNING;
			/* indicate the user of the beginning of the audio session ???*/
			
		}
	}
	else
	{
		trace(TRACE_LEVEL1,stdout,"<callback.c> Ack for an inexistant call-leg !\n");
		/* do not respond*/
	}			

  return 1; /* ok */
}

int OnEvent_New_IncomingBye(sipevent_t *sipevent,int transaction_id)
{
	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	
	trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingBye!\n");

  	call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		ua=(OsipUA*)(call->ua);
		call->status=CALL_TERMINATED;
		call->last_tid=transaction_id;
		/* indicate the user of the end of the call-leg*/
		if (ua->bye!=NULL)
				ua->bye(call,(void*)sipmsg);
		/* respond 200 OK to the client */
		osip_call_leg_respond(call,200,NULL);
		/* then delete the call*/
		osip_call_leg_destroy(call);
	}
	else
	{
		trace(TRACE_LEVEL1,stdout,"<callback.c> Bye for an inexistant call-leg !\n");
		ua=(OsipUA*)list_get(&ua_list,0);
			respond_to_request(ua->config, transaction_id, 481, NULL) ;
		/* return an Inexistant Call-Leg*/
	}			
  	return 1; /* ok */
}

int OnEvent_New_IncomingCancel(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingCancel!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_IncomingInfo(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingInfo!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}
int OnEvent_New_IncomingOptions(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingOptions!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}
int OnEvent_New_IncomingRegister(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingRegister!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}
int OnEvent_New_IncomingPrack(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingPrack!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_IncomingUnknownRequest(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_IncomingUnknownRequest!\n");
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return 1; /* ok */
}


int OnEvent_New_Outgoing1xxResponse(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing1xxResponse!\n");
  //uaapp_annouceoutresponse(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_Outgoing2xxResponse(sipevent_t *sipevent,int transaction_id)
{
  	 trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing2xxResponse!\n");
  	 return 1;
}

int OnEvent_New_Outgoing3xxResponse(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing3xxResponse!\n");
  //uaapp_annouceoutresponse(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_Outgoing4xxResponse(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing4xxResponse!\n");
  //uaapp_annouceoutresponse(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_Outgoing5xxResponse(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing5xxResponse!\n");
  //uaapp_annouceoutresponse(sipevent->sip, transaction_id);
  return 1; /* ok */
}

int OnEvent_New_Outgoing6xxResponse(sipevent_t *sipevent,int transaction_id)
{
  trace(TRACE_LEVEL1,stdout,"<callback.c> OnEvent_New_Outgoing6xxResponse!\n");
  //uaapp_annouceoutresponse(sipevent->sip, transaction_id);
  return 1; /* ok */
}


