#include <string.h>
#include <unistd.h>
#include <signal.h>
#include "OsipUA.h"

void server(void);
void client(void);
void stop_func(int signum);
int invite_cb(OsipCallLeg *cl,void*p);
int bye_cb(OsipCallLeg *cl,void*p);
int invite_accepted_cb(OsipCallLeg *cl,void*p);
int run_cond=1;

static char *mediatype[2]={"AUDIO","VIDEO"};

int main(int argc, char *argv[])
{

	signal(SIGINT,stop_func);
	if (argc!=2)
	{
		printf("usage: osipua_tester [-c] [-s]\n");
		return(-1);
	}
	if (strcmp(argv[1],"-s")==0) server();
	else if (strcmp(argv[1],"-c")==0) client();
	else return(-1);
	return(0);
}
static char *callee="<sip:simon.morlat@www-db.research.bell-labs.com>";
void client(void)
{
	OsipUA *ua;
	OsipCallLeg *call_leg;
	int err;
	ua_params_t params;


	memset(&params,0,sizeof(ua_params_t));
	params.contact="<sip:pablo@217.128.128.75:5062>";
	params.ua_ip4addr="217.128.128.75";
	params.ua_port=5062;
	params.max_calls=1;
	params.dbg=NULL;

	ua=osip_ua_new(&params);
	/* add rtp profile capabilities */
	osip_ua_add_profile(ua,STREAM_AUDIO,3);
	osip_ua_add_profile(ua,STREAM_AUDIO,4);
	osip_ua_set_rtpport(ua,STREAM_AUDIO,7000);
	osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
	osip_ua_signal_connect(ua,"BYE",bye_cb);
	printf("Inviting %s\n",callee);
	call_leg=osip_call_leg_new(ua);
	osip_call_leg_invite(call_leg,callee);

	printf("sleeping...\n");
	while (run_cond) sleep(1);
	//scanf("%i",&err);
	osip_call_leg_bye(call_leg);
	sleep(2);
	printf("exiting...\n");
	osip_ua_destroy(ua);
	exit(1);
}
void server(void)
{
	OsipUA *ua;
	OsipCallLeg *call_leg;
	int err;
	ua_params_t params;

	memset(&params,0,sizeof(ua_params_t));
	params.contact="<sip:simon@127.0.0.1>";
	params.ua_ip4addr="127.0.0.1";
	params.ua_port=5060;
	params.max_calls=1;
	params.dbg=NULL;

	ua=osip_ua_new(&params);
	osip_ua_add_profile(ua,0,4);
	osip_ua_set_rtpport(ua,STREAM_AUDIO,7002);
	osip_ua_signal_connect(ua,"INVITE",invite_cb);
	osip_ua_signal_connect(ua,"BYE",bye_cb);
	printf("sleeping...\n");
	while (run_cond) sleep(1);
	//scanf("%i",&err);
	printf("exiting...\n");
	osip_ua_destroy(ua);
	exit(1);
}

int invite_cb(OsipCallLeg *cl,void*p)
{
	sip_t *sipmsg=(sip_t*)p;
	char *from;
	list_t *streams;
	MediaDesc *desc;
	int i,*prof,osiperr;

	from_2char(sipmsg->from,&from,&osiperr);
	printf("Receiving invite from %s.\n",from);
	sfree(from);
	sleep(1);
	printf("Accepting invite.\n");
	streams=osip_call_leg_accept_invite(cl);
	if (streams==NULL) printf("error accepting invite\n");
	else
	{
		printf("Invite is accepted with the following sdp:\n" );
		for (i=0;i<streams->nb_elt;i++)
		{
			desc=(MediaDesc*)list_get(streams,i);
			prof=list_get(desc->profile_list,0);
			printf("Type: %s   port: %i     profile:%i\n",mediatype[desc->type],desc->rem_port,prof[0]);
		}
	}
	return(0);
}

int bye_cb(OsipCallLeg *cl,void*p)
{
	sip_t *sipmsg=(sip_t*)p;
	char *from;
   int osiperr;
	from_2char(sipmsg->from,&from,&osiperr);
	printf("Receiving bye from %s.\n",from);
	sfree(from);
	exit;
	return(0);
}	

int invite_accepted_cb(OsipCallLeg *cl,void*p)
{
	sip_t *sipmsg=(sip_t*)p;
	int i;
	int *prof;
	MediaDesc *desc;
	list_t *streams;
	
	printf("Invite accepted by remote server!\n");
	streams=osip_call_leg_get_streams(cl);
	if (streams==NULL) printf("No streams properties !!!!\n");
	else
	{
		printf("Invite was accepted with the following sdp:\n" );
		for (i=0;i<streams->nb_elt;i++)
		{
			desc=(MediaDesc*)list_get(streams,i);
			prof=list_get(desc->profile_list,0);
			printf("Type: %s   port: %i     profile:%i\n",mediatype[desc->type],desc->rem_port,prof[0]);
		}
	}
	return(0);
}

void stop_func(int signum)
{
	run_cond=0;
}