/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

callbacks.c -- gtk callbacks, and osipua callbacks.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <applet-widget.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "state-machine.h"
#include "sip_handlers.h"
#include "../audio/audio.h"
#include "../audio/ring.h"
#include "../osipua/OsipUA.h"
#include "../gsmlib/gsm_wrapper.h"
#include "../lpc10-1.5/lpc10_wrapper.h"
#include "io.h"
#include "LinphoneMain.h"
#include "gui_utils.h"

#include "../pixmaps/linphone2.xpm"

#ifdef HAVE_PANEL_PIXEL_SIZE

#endif



/* widgets*/
extern GtkWidget *applet;
extern GtkWidget *main_window;
extern GtkWidget *entry1;
extern GtkWidget *entry2;
extern GtkWidget *appbar1;

GtkWidget *prop1;

/* LinphoneMain object*/
extern LinphoneMain linpmain;


/* messages strings */
extern gchar *ready;
extern gchar *end;
extern gchar *contacting;
extern gchar *connected;
extern gchar *cancel;
extern gchar *contacted;

#ifndef G_THREADS_ENABLED
#error "pas de treads dans la glib!"
#endif

int state=LP_INIT;  /* this integer represents the state machine: it MUST only be changed using a mutex or something
like*/
G_LOCK_DEFINE_STATIC(state);
from_t *rem_url=NULL;
GList *if_list;
static unsigned char selected_row=0;
static unsigned char selected_con_type;




void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (main_window!=NULL)
	{
		gnome_entry_save_history (GNOME_ENTRY (entry1));
		gnome_entry_save_history (GNOME_ENTRY (entry2));
	}
	else printf("Error: main_window=NULL.\n");
	gui_shutdown(&linpmain);
	
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about2;
  about2 = create_about2 ();
  gtk_widget_show (about2);
}


void
on_greenbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *appbar1=(GtkWidget*)user_data;
	gchar *name,*host;
	char sipurl[100];
	char *callee;
	list_t *streams;
	MediaDesc *stream;
	codec_info_t *cod;
	int err,err1;

	G_LOCK(state);
	printf("state=%i\n",state);
	
	if (state==LP_INVITED)
	{
		/*stop ringing */
		stop_ring();
		/* sends a 200 OK */
		streams=osip_call_leg_accept_invite(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
		gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry1))),rem_url->url->username);
		gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry2))),rem_url->url->host);
		state=LP_INVITED_CONFIRMED;
		
		if (streams==NULL)
		{
			printf("No streams !!!");
			exit(1);
		}
		/* prepare rtp session using sdp paramters*/
		stream=(MediaDesc*)list_get(streams,0);
		err=rtp_prepare(linpmain.rtp_conf.audio_rtp_port,stream->rem_ip4addr,stream->rem_port,profile_1(stream),
									linpmain.rtp_conf.jitt_comp,0);
		if (err==0)
		{
			/* chexk if we can open dsp*/
			try_open_dsp();
			cod=linphone_main_get_codec(&linpmain,profile_1(stream));
			/* start the io loop*/
			err1=start_audio_io(cod);
			/* if we fail, it is because of an unopenable audio device*/
			if (err1==-1)
			{
				printf("Cannot start io!!\n");
			}
		}else printf("Failed to create RTP session.\n");
	} 	
	else if (state==LP_INIT)
	{
		/* sends an invite */
		name=gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry1))));
		host=gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry2))));
		if (!((strlen(host)==0) || (strlen(name)==0)))
		{
			snprintf(sipurl,100,_("%s %s@%s"),contacting,name,host);
			gnome_appbar_push(GNOME_APPBAR(appbar1),sipurl);
		
			linpmain.call=osip_call_leg_new(linpmain.ua);
			if (linpmain.call==NULL) printf("error in createcall.\n");
		
			callee=g_malloc(strlen(name)+strlen(host)+10);
			sprintf(callee,"<sip:%s@%s>",name,host);
			err=osip_call_leg_invite(linpmain.call,callee);
			if (err!=0)
			{
				printf("error in invite: %s (%i)\n",strerror(err),err);
				//gnome_app_warning(GNOME_APP(main_window),strerror(err));
				gnome_appbar_push(GNOME_APPBAR(appbar1),ready);
				state=LP_INIT;
			}	
			else state=LP_INVITE;
		}
	}
	printf("invite state=%i\n",state);
	G_UNLOCK(state);
}


void
on_redbutton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *appbar1=(GtkWidget*)user_data;

    G_LOCK(state);
    printf("state end=%i\n",state);
    if (state==LP_INVITED )
    {
       /*stop ringing*/
		stop_ring();
		/* decline the invitation */
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
    }
	else if ((state==LP_INVITED_CONFIRMED) || (state==LP_INVITE_CONFIRMED))
	{
    		 /* sends a bye, the call is terminated */
		stop_audio_io();
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
	}
	else if (state==LP_INVITE)
	{
     /*sends a cancel or a bye ???*/
       stop_ring();
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),cancel);
		state=LP_INIT;
	}
	linpmain.call=NULL;
	if (rem_url!=NULL)
	{
		from_free(rem_url);
		free(rem_url);
		rem_url=NULL;
	}
	G_UNLOCK(state);
	return;
}

int invite_accepted_cb(OsipCallLeg *call,void*p2)
{
	//sip_t *sipmsg= (sip_t *)p2;
	list_t *streams;
	MediaDesc *md;
	int err;
	codec_info_t *cod;

	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
	gdk_threads_leave();
	state=LP_INVITE_CONFIRMED;
	printf("ok state=%i\n",state);
	stop_ring();
	G_UNLOCK(state);
	
	streams=osip_call_leg_get_streams(call);
	if (streams==NULL)
	{
		printf("No streams in a remotely acceted call !!!\n");
		exit(1);
	}
	md=(MediaDesc*)list_get(streams,0);  /* get audio stream descriptor*/
	printf("rem_port=%i\n", md->rem_port);
	err=rtp_prepare(linpmain.rtp_conf.audio_rtp_port,md->rem_ip4addr,md->rem_port,profile_1(md),linpmain.rtp_conf.jitt_comp,0);
	if (err==0)
	{
		gdk_threads_enter();
		try_open_dsp();
		gdk_threads_leave();
		cod=linphone_main_get_codec(&linpmain,profile_1(md));
		err=start_audio_io(cod);
		if (err==-1)
		{
			printf("Failed to start io [2]\n");
		}
	}else printf("Failed to prepare RTP session.\n");
	return 0;
}

int bye_cb(OsipCallLeg *call,void*p2)
{
	stop_ring();  /*stop ringing if we were ringing*/
	stop_audio_io();     /* io of rtp data if we were in communication*/
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),end);
	gdk_threads_leave();
	state=LP_INIT;
	linpmain.call=NULL;
	if (rem_url!=NULL)
	{
		from_free(rem_url);
		free(rem_url);
		rem_url=NULL;
	}
	printf("bye handler: state=%i\n",state);
	G_UNLOCK(state);
	return 0;
}

/* osipua call us in this callback function when an invite arrives*/
int invite_cb(OsipCallLeg *call,void *p2)
{
	sip_t *sipmsg= (sip_t *)p2;
	from_t *from=sipmsg->from;
	char *name;
	char *caller;
	int err;

	G_LOCK(state);
	linpmain.call=call;
	from_2char(from,&caller,&err);
	from_init(&rem_url,&err);
	from_parse(rem_url,caller,&err);
	name=malloc(strlen(caller)+strlen(contacted)+4);
	sprintf(name,"%s %s.",caller,contacted);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),name);
	gdk_threads_leave();
	state=LP_INVITED;
	/* play the ring */
	/* see if dsp is openable*/
	gdk_threads_enter();
	try_open_dsp();
	gdk_threads_leave();
	err=start_ring();
	G_UNLOCK(state);
	if (err==-1)
	{
		printf("Failed to start ring.\n");
	}
	printf("invite handler done.\n");
	free(caller);
	free(name);
	return 0;
}

int informative_cb( OsipCallLeg *call,void *p2)
{
	int err=((int*)p2)[0];
	gdk_threads_enter();
	gdk_threads_leave();
	switch(err)
	{
		case 100:
			break;
		case 180:
			//printf ("RINGBACK STARTED*************************************************************\n");
			start_ringback();
			break;
		case 200:   /* for the moment only REGISTER request followed by 200 OK are said to the user*/
			 gnome_appbar_push(GNOME_APPBAR(appbar1),_("Registration successfull."));
		break;
		default:
			printf("unsupported informative message.\n");
	}
	return 0;
}

int faillure_cb(OsipCallLeg *call,void *p2)
{
	int err=((int*)p2)[0];
	char *reason;
	
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	switch(err)
	{
		case 486:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("User is busy."));
			state=LP_INIT;
		break;
		case 404:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("User cannot be found at given address."));
			state=LP_INIT;
		break;
		case -111:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Remote host was found but refused connection."));
			state=LP_INIT;
		break;
		
		default:
			if (err>0)
			{
				reason=parser_getreason(err);
				gnome_appbar_push(GNOME_APPBAR(appbar1),reason);
				free(reason);
			}
			else printf("faillure_cb: %s\n",strerror(-err));
			state=LP_INIT;
	}
	linpmain.call=NULL;
	gdk_threads_leave();
	G_UNLOCK(state);
	printf("error handler done.\n");
	return 0;
}

void on_propertybox1_apply (GtkButton       *button,
                                        gint pagenum)
{
	int err,ind;
	gchar *tmp;
	GtkAdjustment *adj;
	GList *ltmp;
	GtkWidget *child;
	gboolean state;
	
	switch(pagenum)
	{
	case 0:
	/* get network interface*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"list1");
	ltmp=GTK_LIST(child)->selection;  /* get the selected item*/
	if (ltmp!=NULL)
	{
		ind=g_list_index(if_list,(gpointer)(ltmp->data));
		if (ind>=0)
		{
			printf("selection changed to %i.\n",ind);
			linpmain.net_conf.if_sel=ind;
			linpmain.net_conf.local_addr=linpmain.net_conf.if_addr[ind];
			linpmain.net_conf.if_name=linpmain.net_conf.if_names[ ind ];
			osip_ua_set_ip4addr(linpmain.ua,linpmain.net_conf.local_addr);
		}
		else printf("cannot retrieve selection.\n");
	}
	/* get connection type*/
	linpmain.net_conf.con_type=selected_con_type;
	linpmain.net_conf.bandwidth=bandwidths[selected_con_type];
	linphone_main_apply_codec_change(&linpmain);
	break;
	case 1:
	/* get jitt_comp*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"hscale1");
	linpmain.rtp_conf.jitt_comp=(gtk_range_get_adjustment(GTK_RANGE(child)))->value;
	/* get rtp_port*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"audioport");
	adj=gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(child));
	osip_ua_set_rtpport(linpmain.ua,STREAM_AUDIO,(int)(adj->value));
	linpmain.rtp_conf.audio_rtp_port= (int)(adj->value);
	break;
	case 2:
	/* get sip port*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"spinbutton2");
	adj=gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(child));
	/* if the sip port has changed, we need to restart the ua*/
	if (adj->value!=linpmain.sip_conf.sip_port)
	{
		linphone_main_stop(&linpmain);
		linpmain.sip_conf.sip_port=adj->value;
		printf("Restarting user agent.\n");
		linpmain.sip_conf.sip_port=adj->value;
		err=linphone_main_start(&linpmain);
		if (err<0) printf("Failed to restart user agent!\n");
	}
	child=lookup_widget(prop1,"user_name");
	tmp=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
	if (strlen(tmp)!=0)
	{
		g_free(linpmain.sip_conf.username);
		linpmain.sip_conf.username=tmp;
	}
	child=lookup_widget(prop1,"registrar_checked");
	state=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(child));
	if (!state)
	{
		child=lookup_widget(prop1,"domain_name");
		tmp=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
		if (strlen(tmp)!=0)
		{
			g_free(linpmain.sip_conf.hostname);
			linpmain.sip_conf.hostname=tmp;
		}
		/* unregister if neccessary : REVISIT */
		if (linpmain.ua->registrar!=NULL)
		{
			osip_call_leg_unregister(osip_call_leg_new(linpmain.ua),NULL,NULL);
		}
		/* tell the osip stack that we have no default registrar*/
		osip_ua_set_registrar(linpmain.ua,NULL,NULL);
		/*remove registration timer if present*/
		linphone_main_remove_registration_timer(&linpmain);
	}
	else
	{
		char *passwd;
		/*get password*/
		child=lookup_widget(prop1,"reg_passwd");
		passwd=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
		if (linpmain.sip_conf.reg_conf.passwd!=NULL) g_free(linpmain.sip_conf.reg_conf.passwd);
		linpmain.sip_conf.reg_conf.passwd=passwd;
		 /* get registrar name */
		child=lookup_widget(prop1,"registrar_addr");
		tmp=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
		ind=strlen(tmp);
		if (strlen(tmp)!=0)
		{
			char *tmp2;
			tmp2=g_malloc(ind+5);
			sprintf(tmp2,"sip:%s",tmp);
			if (linpmain.sip_conf.reg_conf.registrar!=NULL) g_free(linpmain.sip_conf.reg_conf.registrar);
			linpmain.sip_conf.reg_conf.registrar=tmp;
			/* tell the osip stack that we have a default registrar*/
			osip_ua_set_registrar(linpmain.ua,tmp2,linpmain.sip_conf.reg_conf.passwd);
			/* set timer for registration*/
			linpmain.sip_conf.reg_conf.use_registrar=1;
			g_free(tmp2);
		}
		else
		{
			/* tell the osip stack that we have no default registrar*/
			osip_ua_set_registrar(linpmain.ua,NULL,NULL);
			/* unset timer if present*/
			linphone_main_remove_registration_timer(&linpmain);
			state=0;  /* the button was checked, but the user did not give us a registrar name!*/
		}
	}
	linpmain.sip_conf.reg_conf.use_registrar=state;
	linphone_main_set_contact(&linpmain);
	if (state)
	{
		linphone_main_set_registration_timer(&linpmain);
		do_registration(NULL);
	}
	
	break;
	case 4:
	/* get sound driver mode*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"combo_entry4");
	tmp=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
	if (strcmp(tmp,"OSS")==0) linpmain.sound_conf.driver_mode=0;
	else if (strcmp(tmp,"ALSA")==0) linpmain.sound_conf.driver_mode=1;
	g_free(tmp);
  	/* init the selected sound driver*/
	switch(linpmain.sound_conf.driver_mode)
  	{
  		case 0:
  			printf ("Init Oss.\n");
  			audio_oss_init();
  			break;
  		case 1:
  		#ifdef ALSA
  			printf("Init Alsa.\n");
  			audio_alsa_init();
  		#else
  			printf("ERROR: Linphone was not compiled with alsa support. Recompile it or use OSS.\n");
  		#endif
  			break;
  	}
  	/* get audio source*/
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"combo_entry5");
  	tmp=gtk_editable_get_chars(GTK_EDITABLE(child),0,-1);
	if (strcmp(tmp,_("micro"))==0) linpmain.sound_conf.source='m';
	else if (strcmp(tmp,_("line"))==0) linpmain.sound_conf.source='l';
	printf("Sound source selection changed to %c.\n",linpmain.sound_conf.source);
	g_free(tmp);
  	set_source(linpmain.sound_conf.source);	
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"autokill_button1");
  	linpmain.sound_conf.autokill=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(child));
  	break;
  	case 3:
  	/* get codec list*/
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"au_codec_list");
  	g_list_free(linpmain.codecs_conf.audio_codecs);
  	linpmain.codecs_conf.audio_codecs=codec_list_retrieve(GTK_CLIST(child));
  	linphone_main_apply_codec_change(&linpmain);
  	break;
  	}
}

gint
on_prop1_close                         (GnomeDialog     *gnomedialog,
                                        gpointer         user_data)
{	
	GtkWidget *child;
	child=gtk_object_get_data(GTK_OBJECT(prop1),"list1");
	gtk_list_remove_items (GTK_LIST(child),if_list);
	gtk_widget_destroy(prop1);
	gnome_appbar_clear_stack( GNOME_APPBAR(appbar1));
	printf("Closed\n");
	return(FALSE);
}




void
on_parametres1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *item,*sel_item=NULL;
	GtkWidget *child,*domain,*registrar;
	int i;
	GtkWidget *optionmenu,*menu;
	GtkItem *menu_item;
	GtkMenuShell *menushell;
	GList *item_list,*elem;
	gint pos=0;
	
	prop1=create_prop1();
	/* set items callbacks for connection_type widget*/
	optionmenu=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"connection_type");
	menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu));
	menushell=GTK_MENU_SHELL(menu);
	elem=item_list=menushell->children;
	while(elem!=NULL)
	{
		menu_item=(GtkItem*)(elem->data);
		gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_connection_type_selected,GINT_TO_POINTER(pos));
		pos++;
		elem=g_list_next(elem);
	}
	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),linpmain.net_conf.con_type);
	selected_con_type=  linpmain.net_conf.con_type;
	/* put the current value of jitt_comp*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"hscale1");
	gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(child)),(gfloat)linpmain.rtp_conf.jitt_comp);
	/* draw the list of interfaces */
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"list1");
	if_list=NULL;
	for(i=0;i<linpmain.net_conf.if_count;i++)
	{
		item=gtk_list_item_new_with_label (linpmain.net_conf.if_names[i]);
		if_list=g_list_append(if_list,(void*)item);
		gtk_widget_show(item);
		if (linpmain.net_conf.if_sel==i) sel_item=item;
	}
	gtk_list_append_items (GTK_LIST(child),if_list);
	if (sel_item!=NULL) gtk_list_item_select(GTK_LIST_ITEM(sel_item));
	else printf("ERROR: Cannot select item!\n");
	/* setup sip section*/
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"spinbutton2");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(child),(gfloat)linpmain.sip_conf.sip_port);
	/* set user's sip address*/
	child=lookup_widget(prop1,"user_name");
	gtk_entry_set_text(GTK_ENTRY(child),linpmain.sip_conf.username);
	domain=lookup_widget(prop1,"domain_name");
	registrar=lookup_widget(prop1,"registrar_addr");
	if (linpmain.sip_conf.reg_conf.registrar!=NULL) gtk_entry_set_text(GTK_ENTRY(registrar),linpmain.sip_conf.reg_conf.registrar);
	child=lookup_widget(prop1,"reg_passwd");
	if (linpmain.sip_conf.reg_conf.passwd!=NULL) gtk_entry_set_text(GTK_ENTRY(child),linpmain.sip_conf.reg_conf.passwd);
	if (linpmain.sip_conf.reg_conf.use_registrar)
	{
		child=lookup_widget(prop1,"registrar_checked");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(child),linpmain.sip_conf.reg_conf.use_registrar);
		/* putting this button active will send the signal "toggled", and so the registrar_toggled is called to do all the work*/
	}
	else gtk_entry_set_text(GTK_ENTRY(domain),linpmain.sip_conf.hostname);
	
	/* display prefered codecs*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"au_codec_list");
	codec_list_construct(GTK_CLIST(child),linpmain.codecs_conf.audio_codecs);
	codec_list_update(GTK_CLIST(child),linpmain.net_conf.bandwidth);
  	
  	/* select used sound drivers*/
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"combo_entry4");
  	switch(linpmain.sound_conf.driver_mode)
	{
		case 0:
			gtk_entry_set_text (GTK_ENTRY (child), _("OSS"));
  			break;
  		case 1:
  			gtk_entry_set_text (GTK_ENTRY (child), _("ALSA"));
  			break;
  	}
  	/* select audio source*/
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"combo_entry5");
  	switch(linpmain.sound_conf.source)
	{
		case 'm':
			gtk_entry_set_text (GTK_ENTRY (child), _("micro"));
  			break;
  		case 'l':
  			gtk_entry_set_text (GTK_ENTRY (child), _("line"));
  			break;
  	}
  	/* display current rtp port */
  	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"audioport");
  	gtk_spin_button_set_value(GTK_SPIN_BUTTON(child),(gfloat)linpmain.rtp_conf.audio_rtp_port);
  	
	/* display state of the autokill button*/
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"autokill_button1");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(child),linpmain.sound_conf.autokill);
	
  	/* set uchanged state to the prop1 box, because gtk_entry_set_text() causes signal "changed"
  	to be sent */
	gnome_property_box_set_state(GNOME_PROPERTY_BOX(prop1),0);
	gtk_widget_show(prop1);
}


void
on_killesd_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	
	kill_sound_daemons();
}



gboolean
on_hscale1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	return(TRUE);
}

void
on_list1_select_child                  (GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_codec_changed                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_cod;
	sel_cod=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_cod,"GSM (13 kbit/s)")==0) temp_config.profile=3;
	else if (strcmp(sel_cod,"LPC10-15 (2.4 kbit/s)")==0) temp_config.profile=4;
	printf("Profile selection changed to %s : %i.\n",sel_cod,temp_config.profile);
	g_free(sel_cod);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_sounddriver_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_sound;
	
	sel_sound=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_sound,"OSS")==0) temp_config.sound_driver=0;
	else if (strcmp(sel_sound,"ALSA")==0) temp_config.sound_driver=1;
	printf("Sound driver selection changed to %s : %i.\n",sel_sound,temp_config.sound_driver);
	g_free(sel_sound);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_user_manual1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry help_ref = { "linphone", "manual.html" };
	gnome_help_display(NULL,&help_ref);
}


gboolean
on_play_vol_button_release_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	set_out_level(vol);
	linpmain.sound_conf.play_lev=vol;
  return FALSE;
}


gboolean
on_rec_vol_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	set_in_level(vol);
	linpmain.sound_conf.rec_lev=vol;
	return FALSE;
}


void
on_prop1_help                          (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry ref = { "linphone", "parameters" };
	gnome_help_pbox_display(NULL,arg1+1,&ref);
}


void
on_audioport_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *rtpport;
	rtpport=gtk_editable_get_chars(editable,0,-1);
	temp_config.rtp_port=atoi(rtpport);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	return;
}


void
on_sipport_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{

	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	return;
}


void
on_source_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_fermer1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
	{
		gtk_widget_destroy(main_window);
		//linphone_config.show_flag=0;
	}
}


void
on_autokill_button1_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	/*
	temp_config.autokill=gtk_toggle_button_get_active(togglebutton);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_app1_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
  	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
  	{
  		printf("Main window destroyed.\n");
  		main_window=NULL;
  	}
	else
	{
	
    	gnome_entry_save_history (GNOME_ENTRY (entry2));
    	gnome_entry_save_history (GNOME_ENTRY (entry1));
    	gui_shutdown(&linpmain);
    	
	}
}

/* this is the callback when the user clicks on the applet*/
gboolean on_applet_clicked(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer ud)
{
	//GtkWidget *main_window=  (GtkWidget *)ud;
	if (main_window==NULL)
	{
		main_window=create_app1();
		gtk_widget_show( main_window);
	}
	else
	{
		gtk_widget_destroy(main_window);
		main_window=NULL;
	}
	return(TRUE);
}


/*this is when the panel size changes*/
void applet_change_pixel_size(GtkWidget *w, int size, gpointer data)
{
	GtkWidget *pixmap,*button;
	
	pixmap=gtk_object_get_data(GTK_OBJECT(applet),"applet_pixmap");	
	button=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(applet),"applet_button");	
	if (button==NULL)
	{
		printf("Cannot find applet button\n");
		return;
	}
	if (pixmap!=NULL) gtk_widget_destroy(pixmap);
	pixmap = gnome_pixmap_new_from_xpm_d_at_size(linphone2_xpm,
							 size-4, size-4);
	gtk_object_set_data(GTK_OBJECT(applet),"applet_pixmap",pixmap);
  	gtk_widget_show(pixmap);
  	gtk_container_add(GTK_CONTAINER(button), pixmap);	
}

void
on_adresse_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *addbook;
	GtkWidget *item;
	GList *item_list=NULL,*elem=linpmain.gui_conf.address_list;
	GtkWidget *list;
	
	addbook=create_address_book();
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	while(elem!=NULL)
	{
		item=gtk_list_item_new_with_label ((gchar*)(elem->data));
		item_list=g_list_append(item_list,item);
		gtk_widget_show(item);
		elem=g_list_next(elem);
	}
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(addbook);
}

void on_address_book_show(GtkWidget *widget,gpointer user_data)
{
	
}



void on_address_list_select_child(GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void on_add_address_clicked(GtkButton *button,gpointer user_data)
{
	gchar *address,*full_address,*displayname;
	GtkWidget *list,*entry;
	GtkWidget *addbook=(GtkWidget*)user_data;
	from_t *sipaddr;
	int err,error,namelen;
	GList *item_list=NULL;
	GtkWidget *item;
	
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_entry");
	address=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"name_entry");
	displayname=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	from_init(&sipaddr,&err);
	error=from_parse(sipaddr,address,&err);
	if (error!=0)
	{
		printf("Sip address is incorrect.\n");
		free(sipaddr);
		g_free(address);
		g_free(displayname);
		return;
	}
	namelen=strlen(displayname);
	full_address=g_malloc(strlen(address)+namelen+6);
	if (namelen!=0)
		sprintf(full_address,"%s <%s>",displayname,address);
	else sprintf(full_address,"%s",address);
	linpmain.gui_conf.address_list=g_list_append( linpmain.gui_conf.address_list, (gpointer)full_address);
	item=gtk_list_item_new_with_label (full_address);
	item_list=g_list_append(item_list,item);
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(item);
	free(sipaddr);
	g_free(address);
	g_free(displayname);
}

void on_remove_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	GList *elem;
	GtkWidget *list;
	gint selected_address;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
	gtk_list_clear_items(GTK_LIST(list), selected_address,selected_address+1);
	elem= g_list_nth(linpmain.gui_conf.address_list,selected_address);
	if (elem==NULL)
	{
		printf("Error: cannot retrieve address\n");
		return;
	}
	g_free(elem->data);
	linpmain.gui_conf.address_list=g_list_remove_link( linpmain.gui_conf.address_list,elem);
	g_list_free_1(elem);
	
}

void on_select_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	gchar *address;
	GtkWidget *list;
	int errc,err;
	gint selected_address;
	from_t *remurl;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
 	address=(gchar*)g_list_nth_data(linpmain.gui_conf.address_list, selected_address);
	from_init(&remurl,&err);
	err=from_parse(remurl,address,&errc);
	if (err!=0) printf("Error parsing address !!\n");
	gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry1))),remurl->url->username);
	gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry2))),remurl->url->host);
	if ( remurl->url->port!=NULL)
	{
		gtk_entry_append_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry2))),":");
		gtk_entry_append_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry2))),remurl->url->port);
	}
	from_free(remurl);
	sfree(remurl);
	gtk_widget_destroy(addbook);
}




void
on_aucodec_up_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row>0)
	{
		gtk_clist_row_move(clist,selected_row,selected_row-1);
		selected_row--;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	}
}


void
on_aucodec_down_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row < clist->rows-1)
	{
		gtk_clist_row_move(clist,selected_row,selected_row+1);
		selected_row++;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	}
}


void
on_aucodec_enable_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Enabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_aucodec_disable_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Disabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_au_codec_list_select_row            (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	selected_row=row;
}


void
on_au_codec_list_row_move              (GtkCList        *clist,
                                        gint             arg1,
                                        gint             arg2,
                                        gpointer         user_data)
{
	//gtk_clist_row_move(clist,arg1,arg2);
	//gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	//printf("moved!");
}

void on_connection_type_selected(GtkItem       *item,
                                        gpointer         user_data)
{
	GtkWidget *child;
	gint pos=GPOINTER_TO_INT(user_data);
	selected_con_type=pos;
	child=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(prop1),"au_codec_list");
	codec_list_update(GTK_CLIST(child),bandwidths[pos]);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_registrar_checked_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *wid;
	gboolean state;
	
	state=gtk_toggle_button_get_active(togglebutton);
	wid=lookup_widget(prop1,"regpasslabel");
	gtk_widget_set_sensitive(wid,state);
	wid=lookup_widget(prop1,"siplabel1");
	gtk_widget_set_sensitive(wid,state);
	wid=lookup_widget(prop1,"reg_passwd");
	gtk_widget_set_sensitive(wid,state);
	wid=lookup_widget(prop1,"registrar_addr");
	gtk_widget_set_sensitive(wid,state);
	wid=lookup_widget(prop1,"domain_name");
	if (!state)
	{
		gtk_editable_set_editable(GTK_EDITABLE(wid),TRUE);
		gtk_entry_set_text(GTK_ENTRY(wid),linpmain.sip_conf.hostname);
	}
	else
	{
		if (linpmain.sip_conf.reg_conf.registrar!=NULL) gtk_entry_set_text(GTK_ENTRY(wid),linpmain.sip_conf.reg_conf.registrar);
		gtk_editable_set_editable(GTK_EDITABLE(wid),FALSE);
	}
}


void
on_user_name_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_domain_name_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}


void
on_registrar_addr_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *tmp;
	GtkWidget *domain;
	/* user's domain name must be also the name of the registrar when it is used*/
	tmp=gtk_editable_get_chars(editable,0,-1);
	domain=lookup_widget(prop1,"domain_name");
	gtk_entry_set_text(GTK_ENTRY(domain),tmp);
	g_free(tmp);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}

/* send a REGISTER request to the default registrar*/
gint do_registration(gpointer data)
{
	/* prevent from emmitting registration for address 127.0.0.1...*/
	if (strcmp(linpmain.net_conf.local_addr,"127.0.0.1")!=0)
	{
		osip_call_leg_register(osip_call_leg_new(linpmain.ua),NULL,NULL,linpmain.sip_conf.reg_conf.expires);
		printf("Registration done.\n");
	}
	else printf("Registrations with local loopback address are forbidden in linphone.\n");
	return TRUE;
}
void
on_reg_passwd_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
}

